@@ -30,6 +30,8 @@
 
 using namespace Akonadi;
 
+// @cond PRIVATE
+
 AgentInstance AgentManagerPrivate::createInstance( const AgentType &type )
 {
   const QString &identifier = mManager->createAgentInstance( type.identifier() );
@@ -41,9 +43,28 @@ AgentInstance AgentManagerPrivate::createInstance( const AgentType &type )
 
 void AgentManagerPrivate::agentTypeAdded( const QString &identifier )
 {
+  // Ignore agent types we already know about, for example because we called
+  // readAgentTypes before.
+  if ( mTypes.contains( identifier ) )
+    return;
+
   const AgentType type = fillAgentType( identifier );
   if ( type.isValid() ) {
     mTypes.insert( identifier, type );
+
+    // The Akonadi ServerManager assumes that the server is up and running as soon
+    // as it knows about at least one agent type.
+    // If we emit the typeAdded() signal here, it therefore thinks the server is
+    // running. However, the AgentManager does not know about all agent types yet,
+    // as the server might still have pending agentTypeAdded() signals, even though
+    // it internally knows all agent types already.
+    // This can cause situations where the client gets told by the ServerManager that
+    // the server is running, yet the client will encounter an error because the
+    // AgentManager doesn't know all types yet.
+    //
+    // Therefore, we read all agent types from the server here so they are known.
+    readAgentTypes();
+
     emit mParent->typeAdded( type );
   }
 }
@@ -61,8 +82,24 @@ void AgentManagerPrivate::agentInstanceAdded( const QString &identifier )
 {
   const AgentInstance instance = fillAgentInstance( identifier );
   if ( instance.isValid() ) {
-    mInstances.insert( identifier, instance );
-    emit mParent->instanceAdded( instance );
+
+    // It is possible that this function is called when the instance is already
+    // in our list we filled initially in the constructor.
+    // This happens when the constructor is called during Akonadi startup, when
+    // the agent processes are not fully loaded and have no D-Bus interface yet.
+    // The server-side agent manager then emits the instance added signal when
+    // the D-Bus interface for the agent comes up.
+    // In this case, we simply notify that the instance status has changed.
+    bool newAgentInstance = !mInstances.contains( identifier );
+    if ( newAgentInstance ) {
+      mInstances.insert( identifier, instance );
+      emit mParent->instanceAdded( instance );
+    }
+    else {
+      mInstances.remove( identifier );
+      mInstances.insert( identifier, instance );
+      emit mParent->instanceStatusChanged( instance );
+    }
   }
 }
 
@@ -94,7 +131,8 @@ void AgentManagerPrivate::agentInstanceProgressChanged( const QString &identifie
 
   AgentInstance &instance = mInstances[ identifier ];
   instance.d->mProgress = progress;
-  instance.d->mStatusMessage = msg;
+  if ( !msg.isEmpty() )
+    instance.d->mStatusMessage = msg;
 
   emit mParent->instanceProgressChanged( instance );
 }
@@ -117,6 +155,16 @@ void AgentManagerPrivate::agentInstanceError( const QString &identifier, const Q
   emit mParent->instanceError( instance, msg );
 }
 
+void AgentManagerPrivate::agentInstanceOnlineChanged( const QString &identifier, bool state )
+{
+  if ( !mInstances.contains( identifier ) )
+    return;
+
+  AgentInstance &instance = mInstances[ identifier ];
+  instance.d->mIsOnline = state;
+  emit mParent->instanceOnline( instance, state );
+}
+
 void AgentManagerPrivate::agentInstanceNameChanged( const QString &identifier, const QString &name )
 {
   if ( !mInstances.contains( identifier ) )
@@ -128,6 +176,17 @@ void AgentManagerPrivate::agentInstanceNameChanged( const QString &identifier, c
   emit mParent->instanceNameChanged( instance );
 }
 
+void AgentManagerPrivate::readAgentTypes()
+{
+  QDBusReply<QStringList> types = mManager->agentTypes();
+  if ( types.isValid() ) {
+    foreach ( const QString &type, types.value() ) {
+      if ( !mTypes.contains( type ) )
+        agentTypeAdded( type );
+    }
+  }
+}
+
 AgentType AgentManagerPrivate::fillAgentType( const QString &identifier ) const
 {
   AgentType type;
@@ -175,7 +234,8 @@ AgentInstance AgentManagerPrivate::fillAgentInstance( const QString &identifier
   AgentInstance instance;
 
   const QString agentTypeIdentifier = mManager->agentInstanceType( identifier );
-  Q_ASSERT_X( mTypes.contains( agentTypeIdentifier ), "fillAgentInstance", "Requests non-existing agent type" );
+  if ( !mTypes.contains( agentTypeIdentifier ) )
+    return instance;
 
   instance.d->mType = mTypes.value( agentTypeIdentifier );
   instance.d->mIdentifier = identifier;
@@ -228,20 +288,30 @@ AgentManager::AgentManager()
            this, SLOT( agentInstanceWarning( const QString&, const QString& ) ) );
   connect( d->mManager, SIGNAL( agentInstanceError( const QString&, const QString& ) ),
            this, SLOT( agentInstanceError( const QString&, const QString& ) ) );
+  connect( d->mManager, SIGNAL(agentInstanceOnlineChanged(QString,bool)),
+           SLOT(agentInstanceOnlineChanged(QString,bool)) );
 
-  const QStringList typeIdentifiers = d->mManager->agentTypes();
-  foreach( const QString &type, typeIdentifiers ) {
-    const AgentType agentType = d->fillAgentType( type );
-    d->mTypes.insert( type, agentType );
-  }
+  if ( d->mManager->isValid() ) {
+    QDBusReply<QStringList> result = d->mManager->agentTypes();
+    if ( result.isValid() ) {
+      foreach( const QString &type, result.value() ) {
+        const AgentType agentType = d->fillAgentType( type );
+        d->mTypes.insert( type, agentType );
+      }
+    }
 
-  const QStringList instanceIdentifiers = d->mManager->agentInstances();
-  foreach( const QString &instance, instanceIdentifiers ) {
-    const AgentInstance agentInstance = d->fillAgentInstance( instance );
-    d->mInstances.insert( instance, agentInstance );
+    result = d->mManager->agentInstances();
+    if ( result.isValid() ) {
+      foreach( const QString &instance, result.value() ) {
+        const AgentInstance agentInstance = d->fillAgentInstance( instance );
+        d->mInstances.insert( instance, agentInstance );
+      }
+    }
   }
 }
 
+// @endcond
+
 AgentManager::~AgentManager()
 {
   delete d;
