@@ -23,11 +23,13 @@
 #include "imapparser_p.h"
 #include "job.h"
 #include "job_p.h"
+#include "servermanager_p.h"
 #include "xdgbasedirs_p.h"
 
 #include <kdebug.h>
 #include <klocale.h>
 
+#include <QCoreApplication>
 #include <QtCore/QDir>
 #include <QtCore/QQueue>
 #include <QtCore/QThreadStorage>
@@ -42,8 +44,6 @@ using namespace Akonadi;
 
 //@cond PRIVATE
 
-static const int minimumProtocolVersion = 2;
-
 void SessionPrivate::startNext()
 {
   QTimer::singleShot( 0, mParent, SLOT(doStartNext()) );
@@ -65,7 +65,14 @@ void SessionPrivate::reconnect()
   }
 }
 
-void SessionPrivate::socketError()
+void SessionPrivate::socketError( QLocalSocket::LocalSocketError error )
+{
+  Q_ASSERT( mParent->sender() == socket );
+  kWarning( 5250 ) << "Socket error occurred:" << socket->errorString();
+  socketDisconnected();
+}
+
+void SessionPrivate::socketDisconnected()
 {
   if ( currentJob )
     currentJob->d_ptr->lostConnection();
@@ -85,7 +92,7 @@ void SessionPrivate::dataReceived()
 
       // handle login response
       if ( parser->tag() == QByteArray("0") ) {
-        if ( parser->data().startsWith( "OK" ) ) {
+        if ( parser->data().startsWith( "OK" ) ) { //krazy:exclude=strings
           connected = true;
           startNext();
         } else {
@@ -102,10 +109,11 @@ void SessionPrivate::dataReceived()
           qint64 tmp = 0;
           ImapParser::parseNumber( parser->data(), tmp, 0, pos + 9 );
           protocolVersion = tmp;
+          Internal::setServerProtocolVersion( tmp );
         }
         kDebug( 5250 ) << "Server protocol version is:" << protocolVersion;
 
-        writeData( "0 LOGIN " + sessionId + '\n' );
+        writeData( "0 LOGIN " + ImapParser::quote( sessionId ) + '\n' );
 
       // work for the current job
       } else {
@@ -154,9 +162,9 @@ void SessionPrivate::doStartNext()
 
 void SessionPrivate::startJob( Job *job )
 {
-  if ( protocolVersion < minimumProtocolVersion ) {
+  if ( protocolVersion < minimumProtocolVersion() ) {
     job->setError( Job::ProtocolVersionMismatch );
-    job->setErrorText( i18n( "Protocol version %1 found, expected at least %2", protocolVersion, minimumProtocolVersion ) );
+    job->setErrorText( i18n( "Protocol version %1 found, expected at least %2", protocolVersion, minimumProtocolVersion() ) );
     job->emitResult();
   } else {
     job->d_ptr->startQueued();
@@ -186,11 +194,23 @@ void SessionPrivate::jobWriteFinished( Akonadi::Job* job )
   startNext();
 }
 
+void SessionPrivate::jobDestroyed(QObject * job)
+{
+  queue.removeAll( static_cast<Akonadi::Job*>( job ) );
+  // ### likely not enough to really cancel already running jobs
+  pipeline.removeAll( static_cast<Akonadi::Job*>( job ) );
+  if ( currentJob == job ) {
+    currentJob = 0;
+    jobRunning = false;
+  }
+}
+
 void SessionPrivate::addJob(Job * job)
 {
   queue.append( job );
   QObject::connect( job, SIGNAL(result(KJob*)), mParent, SLOT(jobDone(KJob*)) );
   QObject::connect( job, SIGNAL(writeFinished(Akonadi::Job*)), mParent, SLOT(jobWriteFinished(Akonadi::Job*)) );
+  QObject::connect( job, SIGNAL(destroyed(QObject*)), mParent, SLOT(jobDestroyed(QObject*)) );
   startNext();
 }
 
@@ -211,10 +231,12 @@ Session::Session(const QByteArray & sessionId, QObject * parent) :
     QObject( parent ),
     d( new SessionPrivate( this ) )
 {
-  if ( !sessionId.isEmpty() )
+  if ( !sessionId.isEmpty() ) {
     d->sessionId = sessionId;
-  else
-    d->sessionId = QByteArray::number( qrand() );
+  } else {
+    d->sessionId = QCoreApplication::instance()->applicationName().toUtf8()
+        + '-' + QByteArray::number( qrand() );
+  }
 
   d->connected = false;
   d->theNextTag = 1;
@@ -236,8 +258,8 @@ Session::Session(const QByteArray & sessionId, QObject * parent) :
   // should check connection method
   d->socket = new QLocalSocket( this );
 
-  connect( d->socket, SIGNAL(disconnected()), SLOT(socketError()) );
-  connect( d->socket, SIGNAL(error(QLocalSocket::LocalSocketError)), SLOT(socketError()) );
+  connect( d->socket, SIGNAL(disconnected()), SLOT(socketDisconnected()) );
+  connect( d->socket, SIGNAL(error(QLocalSocket::LocalSocketError)), SLOT(socketError(QLocalSocket::LocalSocketError)) );
   connect( d->socket, SIGNAL(readyRead()), SLOT(dataReceived()) );
   d->reconnect();
 }
