@@ -31,8 +31,11 @@ class ItemAppendTest : public QObject
     void testItemAppend();
     void testContent_data();
     void testContent();
+    void testNewMimetype();
     void testIllegalAppend();
     void testMultipartAppend();
+    void testItemSize_data();
+    void testItemSize();
 };
 
 
