@@ -709,32 +709,9 @@ class KCal::RecurrenceRule::Private
         mAllDay( false )
     {}
 
-    Private( RecurrenceRule *parent, const Private &p )
-      : mParent( parent ),
-        mRRule( p.mRRule ),
-        mPeriod( p.mPeriod ),
-        mDateStart( p.mDateStart ),
-        mFrequency( p.mFrequency ),
-        mDuration( p.mDuration ),
-        mDateEnd( p.mDateEnd ),
-
-        mBySeconds( p.mBySeconds ),
-        mByMinutes( p.mByMinutes ),
-        mByHours( p.mByHours ),
-        mByDays( p.mByDays ),
-        mByMonthDays( p.mByMonthDays ),
-        mByYearDays( p.mByYearDays ),
-        mByWeekNumbers( p.mByWeekNumbers ),
-        mByMonths( p.mByMonths ),
-        mBySetPos( p.mBySetPos ),
-        mWeekStart( p.mWeekStart ),
-
-        mIsReadOnly( p.mIsReadOnly ),
-        mAllDay( p.mAllDay )
-    {
-        setDirty();
-    }
+    Private( RecurrenceRule *parent, const Private &p );
 
+    Private &operator=( const Private &other );
     bool operator==( const Private &other ) const;
     void clear();
     void setDirty();
@@ -751,7 +728,6 @@ class KCal::RecurrenceRule::Private
                                // unless it matches the rule)
     uint mFrequency;
     /** how often it recurs:
-          -1 means infinitely,
            0 means an explicit end date,
            positive values give the number of occurrences */
     int mDuration;
@@ -784,6 +760,65 @@ class KCal::RecurrenceRule::Private
     uint mTimedRepetition;  // repeats at a regular number of seconds interval, or 0
 };
 
+RecurrenceRule::Private::Private( RecurrenceRule *parent, const Private &p )
+  : mParent( parent ),
+    mRRule( p.mRRule ),
+    mPeriod( p.mPeriod ),
+    mDateStart( p.mDateStart ),
+    mFrequency( p.mFrequency ),
+    mDuration( p.mDuration ),
+    mDateEnd( p.mDateEnd ),
+
+    mBySeconds( p.mBySeconds ),
+    mByMinutes( p.mByMinutes ),
+    mByHours( p.mByHours ),
+    mByDays( p.mByDays ),
+    mByMonthDays( p.mByMonthDays ),
+    mByYearDays( p.mByYearDays ),
+    mByWeekNumbers( p.mByWeekNumbers ),
+    mByMonths( p.mByMonths ),
+    mBySetPos( p.mBySetPos ),
+    mWeekStart( p.mWeekStart ),
+
+    mIsReadOnly( p.mIsReadOnly ),
+    mAllDay( p.mAllDay )
+{
+    setDirty();
+}
+
+RecurrenceRule::Private &RecurrenceRule::Private::operator=( const Private &p )
+{
+  // check for self assignment
+  if ( &p == this ) {
+    return *this;
+  }
+
+  mRRule = p.mRRule;
+  mPeriod = p.mPeriod;
+  mDateStart = p.mDateStart;
+  mFrequency = p.mFrequency;
+  mDuration = p.mDuration;
+  mDateEnd = p.mDateEnd;
+
+  mBySeconds = p.mBySeconds;
+  mByMinutes = p.mByMinutes;
+  mByHours = p.mByHours;
+  mByDays = p.mByDays;
+  mByMonthDays = p.mByMonthDays;
+  mByYearDays = p.mByYearDays;
+  mByWeekNumbers = p.mByWeekNumbers;
+  mByMonths = p.mByMonths;
+  mBySetPos = p.mBySetPos;
+  mWeekStart = p.mWeekStart;
+
+  mIsReadOnly = p.mIsReadOnly;
+  mAllDay = p.mAllDay;
+
+  setDirty();
+
+  return *this;
+}
+
 bool RecurrenceRule::Private::operator==( const Private &r ) const
 {
   return
@@ -863,6 +898,18 @@ bool RecurrenceRule::operator==( const RecurrenceRule &r ) const
   return *d == *r.d;
 }
 
+RecurrenceRule &RecurrenceRule::operator=( const RecurrenceRule &r )
+{
+  // check for self assignment
+  if ( &r == this ) {
+    return *this;
+  }
+
+  *d = *r.d;
+
+  return *this;
+}
+
 void RecurrenceRule::addObserver( RuleObserver *observer )
 {
   if ( !d->mObservers.contains( observer ) ) {
@@ -1694,6 +1741,8 @@ DateTimeList RecurrenceRule::timesInInterval( const KDateTime &dtStart,
     KDateTime dt = start.addSecs( d->mTimedRepetition - n );
     if ( dt < enddt ) {
       n = static_cast<int>( ( dt.secsTo_long( enddt ) - 1 ) / d->mTimedRepetition ) + 1;
+      // limit n by a sane value else we can "explode".
+      n = qMin( n, LOOP_LIMIT );
       for ( int i = 0;  i < n;  dt = dt.addSecs( d->mTimedRepetition ), ++i ) {
         result += dt;
       }
@@ -1789,7 +1838,9 @@ Constraint RecurrenceRule::Private::getPreviousValidDateInterval( const KDateTim
   case rSecondly:
     periods = static_cast<int>( start.secsTo_long( toDate ) / modifier );
     // round it down to the next lower multiple of frequency:
-    periods = ( periods / mFrequency ) * mFrequency;
+    if ( mFrequency > 0 ) {
+      periods = ( periods / mFrequency ) * mFrequency;
+    }
     nextValid = start.addSecs( modifier * periods );
     break;
   case rWeekly:
@@ -1799,7 +1850,9 @@ Constraint RecurrenceRule::Private::getPreviousValidDateInterval( const KDateTim
   case rDaily:
     periods = start.daysTo( toDate ) / modifier;
     // round it down to the next lower multiple of frequency:
-    periods = ( periods / mFrequency ) * mFrequency;
+    if ( mFrequency > 0 ) {
+      periods = ( periods / mFrequency ) * mFrequency;
+    }
     nextValid = start.addDays( modifier * periods );
     break;
   case rMonthly:
@@ -1807,7 +1860,9 @@ Constraint RecurrenceRule::Private::getPreviousValidDateInterval( const KDateTim
     periods = 12 * ( toDate.date().year() - start.date().year() ) +
               ( toDate.date().month() - start.date().month() );
     // round it down to the next lower multiple of frequency:
-    periods = ( periods / mFrequency ) * mFrequency;
+    if ( mFrequency > 0 ) {
+      periods = ( periods / mFrequency ) * mFrequency;
+    }
     // set the day to the first day of the month, so we don't have problems
     // with non-existent days like Feb 30 or April 31
     start.setDate( QDate( start.date().year(), start.date().month(), 1 ) );
@@ -1816,7 +1871,9 @@ Constraint RecurrenceRule::Private::getPreviousValidDateInterval( const KDateTim
   case rYearly:
     periods = ( toDate.date().year() - start.date().year() );
     // round it down to the next lower multiple of frequency:
-    periods = ( periods / mFrequency ) * mFrequency;
+    if ( mFrequency > 0 ) {
+      periods = ( periods / mFrequency ) * mFrequency;
+    }
     nextValid.setDate( start.date().addYears( periods ) );
     break;
   default:
@@ -1853,7 +1910,7 @@ Constraint RecurrenceRule::Private::getNextValidDateInterval( const KDateTime &d
   case rSecondly:
     periods = static_cast<int>( start.secsTo_long( toDate ) / modifier );
     periods = qMax( 0L, periods );
-    if ( periods > 0 ) {
+    if ( periods > 0 && mFrequency > 0 ) {
       periods += ( mFrequency - 1 - ( ( periods - 1 ) % mFrequency ) );
     }
     nextValid = start.addSecs( modifier * periods );
@@ -1866,7 +1923,7 @@ Constraint RecurrenceRule::Private::getNextValidDateInterval( const KDateTime &d
   case rDaily:
     periods = start.daysTo( toDate ) / modifier;
     periods = qMax( 0L, periods );
-    if ( periods > 0 ) {
+    if ( periods > 0 && mFrequency > 0 ) {
       periods += ( mFrequency - 1 - ( ( periods - 1 ) % mFrequency ) );
     }
     nextValid = start.addDays( modifier * periods );
@@ -1876,7 +1933,7 @@ Constraint RecurrenceRule::Private::getNextValidDateInterval( const KDateTime &d
     periods = 12 * ( toDate.date().year() - start.date().year() ) +
               ( toDate.date().month() - start.date().month() );
     periods = qMax( 0L, periods );
-    if ( periods > 0 ) {
+    if ( periods > 0 && mFrequency > 0 ) {
       periods += ( mFrequency - 1 - ( ( periods - 1 ) % mFrequency ) );
     }
     // set the day to the first day of the month, so we don't have problems
@@ -1888,7 +1945,7 @@ Constraint RecurrenceRule::Private::getNextValidDateInterval( const KDateTime &d
   case rYearly:
     periods = ( toDate.date().year() - start.date().year() );
     periods = qMax( 0L, periods );
-    if ( periods > 0 ) {
+    if ( periods > 0 && mFrequency > 0 ) {
       periods += ( mFrequency - 1 - ( ( periods - 1 ) % mFrequency ) );
     }
     nextValid.setDate( start.date().addYears( periods ) );
@@ -2043,6 +2100,7 @@ QString dumpTime( const KDateTime &dt )
   }
   return result;
 #else
+  Q_UNUSED( dt );
   return QString();
 #endif
 }
