@@ -30,6 +30,7 @@
 */
 
 #include "todo.h"
+#include "incidenceformatter.h"
 
 #include <kglobal.h>
 #include <klocale.h>
@@ -108,6 +109,11 @@ Todo *Todo::clone()
 
 Todo &Todo::operator=( const Todo &other )
 {
+  // check for self assignment
+  if ( &other == this ) {
+    return *this;
+  }
+
   Incidence::operator=( other );
   d->init( *other.d );
   return *this;
@@ -116,7 +122,7 @@ Todo &Todo::operator=( const Todo &other )
 bool Todo::operator==( const Todo &todo ) const
 {
   return
-    static_cast<const Incidence &>( *this ) == static_cast<const Incidence &>( todo ) &&
+    Incidence::operator==( todo ) &&
     dtDue() == todo.dtDue() &&
     hasDueDate() == todo.hasDueDate() &&
     hasStartDate() == todo.hasStartDate() &&
@@ -154,8 +160,6 @@ void Todo::setDtDue( const KDateTime &dtDue, bool first )
     setDtStart( dtDue );
   }
 
-  //kDebug() << "setDtDue says date is" << d->mDtDue.toString();
-
   /*const Alarm::List& alarms = alarms();
   for (Alarm *alarm = alarms.first(); alarm; alarm = alarms.next())
     alarm->setAlarmStart(d->mDtDue);*/
@@ -184,10 +188,11 @@ QString Todo::dtDueTimeStr( bool shortfmt, const KDateTime::Spec &spec ) const
       timeZone = ' ' + spec.timeZone().name();
     }
 
-    return KGlobal::locale()->formatTime( dtDue( !recurs() ).toTimeSpec( spec ).time(), !shortfmt )
-      + timeZone;
+    return KGlobal::locale()->formatTime(
+      dtDue( !recurs() ).toTimeSpec( spec ).time(), !shortfmt ) + timeZone;
   } else {
-    return KGlobal::locale()->formatTime( dtDue( !recurs() ).time(), !shortfmt );
+    return KGlobal::locale()->formatTime(
+      dtDue( !recurs() ).time(), !shortfmt );
   }
 }
 
@@ -202,8 +207,7 @@ QString Todo::dtDueDateStr( bool shortfmt, const KDateTime::Spec &spec ) const
 
     return KGlobal::locale()->formatDate(
       dtDue( !recurs() ).toTimeSpec( spec ).date(),
-      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) )
-      + timeZone;
+      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) ) + timeZone;
   } else {
     return KGlobal::locale()->formatDate(
       dtDue( !recurs() ).date(),
@@ -214,7 +218,7 @@ QString Todo::dtDueDateStr( bool shortfmt, const KDateTime::Spec &spec ) const
 QString Todo::dtDueStr( bool shortfmt, const KDateTime::Spec &spec ) const
 {
   if ( allDay() ) {
-    return dtDueDateStr( shortfmt, spec );
+    return IncidenceFormatter::dateToString( dtDue(), shortfmt, spec );
   }
 
   if ( spec.isValid() ) {
@@ -226,8 +230,7 @@ QString Todo::dtDueStr( bool shortfmt, const KDateTime::Spec &spec ) const
 
     return KGlobal::locale()->formatDateTime(
       dtDue( !recurs() ).toTimeSpec( spec ).dateTime(),
-      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) )
-      + timeZone;
+      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) ) + timeZone;
   } else {
     return  KGlobal::locale()->formatDateTime(
       dtDue( !recurs() ).dateTime(),
@@ -283,7 +286,7 @@ KDateTime Todo::dtStart( bool first ) const
     return KDateTime();
   }
   if ( recurs() && !first ) {
-    return d->mDtRecurrence.addDays( dtDue( first ).daysTo( IncidenceBase::dtStart() ) );
+    return d->mDtRecurrence.addDays( dtDue( true ).daysTo( IncidenceBase::dtStart() ) );
   } else {
     return IncidenceBase::dtStart();
   }
@@ -308,16 +311,17 @@ QString Todo::dtStartTimeStr( bool shortfmt, bool first, const KDateTime::Spec &
       timeZone = ' ' + spec.timeZone().name();
     }
 
-    return KGlobal::locale()->formatTime( dtStart( first ).toTimeSpec( spec ).time(), !shortfmt )
-      + timeZone;
+    return KGlobal::locale()->formatTime(
+      dtStart( first ).toTimeSpec( spec ).time(), !shortfmt ) + timeZone;
   } else {
-    return KGlobal::locale()->formatTime( dtStart( first ).time(), !shortfmt );
+    return KGlobal::locale()->formatTime(
+      dtStart( first ).time(), !shortfmt );
   }
 }
 
 QString Todo::dtStartTimeStr( bool shortfmt, const KDateTime::Spec &spec ) const
 {
-  return Incidence::dtStartTimeStr( shortfmt, spec );
+  return IncidenceFormatter::timeToString( dtStart(), shortfmt, spec );
 }
 
 QString Todo::dtStartDateStr( bool shortfmt, bool first, const KDateTime::Spec &spec ) const
@@ -329,26 +333,25 @@ QString Todo::dtStartDateStr( bool shortfmt, bool first, const KDateTime::Spec &
       timeZone = ' ' + spec.timeZone().name();
     }
 
-    return
-      KGlobal::locale()->formatDate( dtStart( first ).toTimeSpec( spec ).date(),
-                                   ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) )
-      + timeZone;
+    return KGlobal::locale()->formatDate(
+      dtStart( first ).toTimeSpec( spec ).date(),
+      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) ) + timeZone;
   } else {
-    return
-      KGlobal::locale()->formatDate( dtStart( first ).date(),
-                                   ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) );
+    return KGlobal::locale()->formatDate(
+      dtStart( first ).date(),
+      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) );
   }
 }
 
 QString Todo::dtStartDateStr( bool shortfmt, const KDateTime::Spec &spec ) const
 {
-  return Incidence::dtStartDateStr( shortfmt, spec );
+  return IncidenceFormatter::dateToString( dtStart(), shortfmt, spec );
 }
 
 QString Todo::dtStartStr( bool shortfmt, bool first, const KDateTime::Spec &spec ) const
 {
   if ( allDay() ) {
-    return dtStartDateStr( shortfmt, spec );
+    return IncidenceFormatter::dateToString( dtStart(), shortfmt, spec );
   }
 
   if ( spec.isValid() ) {
@@ -358,20 +361,37 @@ QString Todo::dtStartStr( bool shortfmt, bool first, const KDateTime::Spec &spec
       timeZone = ' ' + spec.timeZone().name();
     }
 
-    return
-      KGlobal::locale()->formatDateTime( dtStart( first ).toTimeSpec( spec ).dateTime(),
-                                       ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) )
-      + timeZone;
+    return KGlobal::locale()->formatDateTime(
+      dtStart( first ).toTimeSpec( spec ).dateTime(),
+      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) ) + timeZone;
   } else {
-    return
-      KGlobal::locale()->formatDateTime( dtStart( first ).dateTime(),
-                                       ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) );
+    return KGlobal::locale()->formatDateTime(
+      dtStart( first ).dateTime(),
+      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) );
   }
 }
 
 QString Todo::dtStartStr( bool shortfmt, const KDateTime::Spec &spec ) const
 {
-  return Incidence::dtStartStr( shortfmt, spec );
+  if ( allDay() ) {
+    return IncidenceFormatter::dateToString( dtStart(), shortfmt, spec );
+  }
+
+  if ( spec.isValid() ) {
+
+    QString timeZone;
+    if ( spec.timeZone() != KSystemTimeZones::local() ) {
+      timeZone = ' ' + spec.timeZone().name();
+    }
+
+    return KGlobal::locale()->formatDateTime(
+      dtStart().toTimeSpec( spec ).dateTime(),
+      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) ) + timeZone;
+  } else {
+    return KGlobal::locale()->formatDateTime(
+      dtStart().dateTime(),
+      ( shortfmt ? KLocale::ShortDate : KLocale::LongDate ) );
+  }
 }
 
 bool Todo::isCompleted() const
@@ -508,7 +528,9 @@ bool Todo::Private::recurTodo( Todo *todo )
       while ( !todo->recursAt( nextDate ) ||
               nextDate <= KDateTime::currentUtcDateTime() ) {
 
-        if ( !nextDate.isValid() || nextDate > endDateTime ) {
+        if ( !nextDate.isValid() ||
+             ( nextDate > endDateTime && r->duration() != -1 ) ) {
+
           return false;
         }
 
