@@ -57,8 +57,7 @@ extern const uchar eTextMap[16];
 
 inline bool isOfSet( const uchar map[16], unsigned char ch )
 {
-  Q_ASSERT( ch < 128 );
-  return map[ ch/8 ] & 0x80 >> ch%8;
+  return ( ch < 128 ) && ( map[ ch/8 ] & 0x80 >> ch%8 );
 }
 inline bool isSpecial( char ch )
 {
@@ -244,6 +243,14 @@ KMIME_EXPORT extern void removeQuots( QString &str );
 */
 KMIME_EXPORT extern void addQuotes( QByteArray &str, bool forceQuotes );
 
+/**
+ * Similar to the above function. Instead of trying to balance the Bidi chars, it outright
+ * removes them from the string.
+ * 
+ * Reason: KHTML seems to ignore the PDF character, so adding them doesn't fix things :(
+ */
+KMIME_EXPORT QString removeBidiControlChars( const QString &input );
+
 } // namespace KMime
 
 #endif /* __KMIME_UTIL_H__ */
